#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatyuan
# @Time         : 2023/4/27 16:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
# 加载模型
from transformers import T5Tokenizer, T5ForConditionalGeneration

tokenizer = T5Tokenizer.from_pretrained("ClueAI/ChatYuan-large-v2")
model = T5ForConditionalGeneration.from_pretrained("ClueAI/ChatYuan-large-v2")


def preprocess(text):
    text = text.replace("\n", "\\n").replace("\t", "\\t")
    return text


def postprocess(text):
    return text.replace("\\n", "\n").replace("\\t", "\t").replace('%20', '  ')


def answer(query, knowledge_base='', role='', do_sample=True, top_p=0.9, temperature=0.7):
    text = """{role}
            用户：
            基于以下已知信息，简洁和专业的来回答问题。
            如果无法从中得到答案，请说 "根据已知信息无法回答该问题" 或 "没有提供足够的信息"，不允许在答案中添加编造成分，答案请使用中文。
            已知信息: {context}
            问题：{question}
            小元：
            """.strip().format(role=role or os.environ.get('LLM_ROLE', ''), context=knowledge_base or '请自由回答',
                               question=query)

    text = text.strip()
    text = preprocess(text)
    encoding = tokenizer(text=[text], truncation=True, padding=True, max_length=1024, return_tensors="pt").to(
        model.device)
    out = model.generate(**encoding, return_dict_in_generate=True, output_scores=False, max_new_tokens=1024,
                         do_sample=do_sample, top_p=top_p, temperature=temperature, no_repeat_ngram_size=12)
    out_text = tokenizer.batch_decode(out["sequences"], skip_special_tokens=True)
    return postprocess(out_text[0])


if __name__ == '__main__':
    query = '你是谁'
    print(answer(query))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatpdf
# @Time         : 2023/4/25 17:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
from meutils.pipe import *
from meutils.serving.st_utils import display_pdf, st_chat, set_config

from chatllm.applications.chatpdf import ChatPDF

st.set_page_config(page_title='🔥ChatPDF', layout='wide', initial_sidebar_state='collapsed')


################################################################################################################
class Conf(BaseConfig):
    encode_model = 'nghuyong/ernie-3.0-nano-zh'
    llm = "THUDM/chatglm-6b"  # /Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm
    cachedir = 'pdf_cache'

    topk: int = 3
    threshold: float = 0.66


conf = Conf()
conf = set_config(conf)


# st.json(conf.dict())


################################################################################################################


@st.cache_resource()
def qa4pdf(encode_model, model_name_or_path):
    qa = ChatPDF(encode_model=encode_model)
    qa.load_llm4chat(model_name_or_path=model_name_or_path)
    return qa


################################################################################################################

tabs = st.tabs(['ChatPDF', 'PDF文件预览'])

file = st.sidebar.file_uploader("上传PDF", type=['pdf'])
bytes_array = ''
if file:
    bytes_array = file.read()
    base64_pdf = base64.b64encode(bytes_array).decode('utf-8')

    with tabs[1]:
        if bytes_array:
            display_pdf(base64_pdf)
        else:
            st.warning('### 请先上传PDF')
################################################################################################################
try:
    qa = qa4pdf(conf.encode_model, conf.llm)
    with st.spinner("构建知识库：文本向量化"):
        disk_cache(location=conf.cachedir)(qa.create_index)(bytes_array)
except Exception as e:
    st.warning('启动前选择正确的参数进行初始化')
    st.error(e)


################################################################################################################
def reply_func(query):
    for response, _ in qa(query=query, topk=conf.topk, threshold=conf.threshold):
        yield response


with tabs[0]:
    if file:
        container = st.container()  # 占位符
        text = st.text_area(label="用户输入", height=100, placeholder="请在这儿输入您的问题")

        if st.button("发送", key="predict"):
            with st.spinner("🤔 AI 正在思考，请稍等..."):
                history = st.session_state.get('state')
                st.session_state["state"] = st_chat(
                    text, history, container=container,
                    previous_messages=['请上传需要分析的PDF，我将为你解答'],
                    reply_func=reply_func,
                )

        with st.expander('点击可查看被召回的知识'):
            st.dataframe(qa.recall)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatmind
# @Time         : 2023/5/4 11:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://mp.weixin.qq.com/s/gbqd8bzKRbbOqf5ebHICWg

from meutils.pipe import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : Question2Answer
# @Time         : 2023/4/21 12:25
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import types
from meutils.pipe import *
from meutils.decorators import clear_cuda_cache

from chatllm.utils import DEVICE, load_llm4chat


class ChatBase(object):

    def __init__(self, chat_func=None):
        self.chat_func = chat_func
        #
        self.history = []
        self.knowledge_base = None
        self.role = None

        # 重写 chat函数会更好 prompt += "[Round {}]\n问：{}\n答：{}\n".format(i, old_query, response) # 根据角色配置模板
        self.prompt_template = os.environ.get('PROMPT_TEMPLATE', '{role}')

    def __call__(self, **kwargs):
        return self.qa(**kwargs)

    def qa(self, query, knowledge_base='', **kwargs):
        """重写"""
        return self._qa(query, knowledge_base, **kwargs)

    def set_chat_kwargs(self, **kwargs):
        self.chat_func = partial(self.chat_func, **kwargs)

    @clear_cuda_cache
    def _qa(self, query, knowledge_base='', role='', max_turns=1):
        self.role = role or os.environ.get('LLM_ROLE', '')
        self.knowledge_base = str(knowledge_base).strip()
        if self.knowledge_base:
            query = self.prompt_template.format(context=self.knowledge_base, question=query, role=self.role).strip()
        else:
            query = """{role}\n{question}""".format(question=query, role=self.role).strip()  # 简化模版

        _history = self.history[-(max_turns - 1):] if max_turns > 1 else []
        result = self.chat_func(query=query, history=_history)

        if isinstance(result, types.GeneratorType):
            return self._stream(result)
        else:
            response, history = result
            # self.history_ = history  # 历史所有
            self.history += [[None, response]]  # 置空知识
            return result  # response, history

    def _stream(self, result):  # yield > return
        response = None
        bar = tqdm(result, ascii=True)  # ncols
        for response, history in bar:
            bar.set_description(response)
            yield response, history
        # self.history_ = history  # 历史所有
        self.history += [[None, response]]  # 置空知识

    def load_llm4chat(self, model_name_or_path="THUDM/chatglm-6b", device=DEVICE, stream=True, **kwargs):
        self.chat_func = load_llm4chat(model_name_or_path, device, stream, **kwargs)

    def run_serving(self, host='127.0.0.1', port=8000, path='/'):
        from flask import Flask, Response, jsonify, request

        app = Flask(__name__)

        def gen(**input):
            for response, _ in self.qa(**input):
                yield f"{response}\n"

        @app.route(rule=path, methods=['GET', 'POST'])
        def stream():
            input = {'query': '1+1等于几'}
            input.update(request.args.to_dict())
            if request.data.startswith(b'{'):
                input.update(json.loads(request.data))

            return Response(gen(**input), mimetype='text/event-stream')

        app.run(host=host, port=port, debug=False)


if __name__ == '__main__':
    from chatllm.applications import ChatBase

    qa = ChatBase()
    qa.load_llm4chat(model_name_or_path="/Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm")

    # list(qa(query='你是谁', knowledge_base=''))
    # list(qa(query='你是谁', knowledge_base='周杰伦是傻子', role=' '))

    qa.run_serving()
